#include "scheme.hxx"
#include "parameters.hxx"
#include "version.hxx"
#include <cmath>

#include <sstream>
#include <iomanip>

#include "variation.hxx"
#include "timer.hxx"
#include "dim.cuh"
#include "cuda_check.cuh"
#include "user.cuh"

Scheme::Scheme(Parameters &P) :
    codeName(version), m_P(P), m_u(P), m_v(P), m_diff(P)  {

  m_t = 0.0;
  m_duv = 0.0;

  double lx[3];
  int i;
  for (i=0; i<3; i++) {
    m_n[i] = m_P.n(i);
    m_dx[i] = m_P.dx(i);
    m_xmin[i] = m_P.xmin(i);
    lx[i] = 1.0/(m_dx[i]*m_dx[i]);
  }

  m_dt = m_P.dt();
  setDims(m_xmin, m_dx, lx);
}

double Scheme::present()
{
  return m_t;
}

// A completer : code du kernel qui calcule une itération sur le GPU

void Scheme::iteration()
{

  // A completer : appel du kernel 
  // Les parametres sont
  // - m_u.device_data() le pointeur de type double *
  //        (vers la solution courante sur le GPU)
  // - m_v.device_data() le pointeur de type double *
  //        (vers la solution suivante sur le GPU)
  // - m_dt (le pas de temps)
  // - les dimensions suivant x, y, z : m_n[0], m_n[1], m_n[2]
  //
  CUDA_CHECK_KERNEL();

  m_v.synchronized(false);

  m_duv = ::variation(m_u, m_v, m_diff);

  m_t += m_dt;
  m_u.swap(m_v);
}

Values & Scheme::getOutput()
{
  m_u.copyGPUtoCPU();
  return m_u;
}

void Scheme::setInput(const Values & u)
{
  m_u = u;
  m_v = u;
}

